@@ -114,7 +114,8 @@
 (defvar egg:*henkan-open*  "|" "*$BJQ49$N;OE@$r<($9J8;zNs(B")
 (defvar egg:*henkan-close* "|" "*$BJQ49$N=*E@$r<($9J8;zNs(B")
 
-(defvar egg:henkan-mode-in-use nil)
+(make-variable-buffer-local
+ (defvar egg:henkan-mode-in-use nil "buffer $B$,JQ49Cf$N;~(B t"))
 
 ;;; ----------------------------------------------------------------
 ;;;	$B0J2<$N(B its mode $B4X78$N4X?t$O!"(Begg.el $B$GDj5A$5$l$F$$$k$,!"(B
@@ -184,9 +185,10 @@ After changing, its:select-mode-hook is called."
 (defun read-current-its-string (prompt &optional initial-input henkan)
   (let ((egg:fence-buffer (window-buffer (minibuffer-window)))
         (old-its-map its:*current-map*)
-        (minibuffer-exit-hook
-         (append minibuffer-exit-hook
-                 '((lambda () (use-local-map minibuffer-local-map))))))
+        ;;(minibuffer-exit-hook
+        ;; (append minibuffer-exit-hook
+        ;;         '((lambda () (use-local-map minibuffer-local-map)))))
+	)
   (save-excursion
     (set-buffer egg:fence-buffer)
     (setq egg:*input-mode* t
@@ -351,8 +353,8 @@ Conversion commands
       (register-notify "$B<-=q9`L\!X(B%s$B!Y(B(%s: %s)$B$r(B%s$B$KEPO?$7$^$9!#(B")
       (cannot-remove "$B%7%9%F%`<-=q9`L\$O:o=|$G$-$^$;$s!#(B")
       (enter-hindo "$BIQEY$rF~$l$F2<$5$$(B: ")
-      (remove-notify "$B<-=q9`L\(B%s(%s)$B$r(B%s$B$+$i:o=|$7$^$9!#(B")
-      (removed "$B<-=q9`L\(B%s(%s)$B$r(B%s$B$+$i:o=|$7$^$7$?!#(B")
+      (remove-notify "$B<-=q9`L\!X(B%s$B!Y(B(%s)$B$r(B%s$B$+$i:o=|$7$^$9!#(B")
+      (removed "$B<-=q9`L\!X(B%s$B!Y(B(%s)$B$r(B%s$B$+$i:o=|$7$^$7$?!#(B")
       (jishomei "$B<-=qL>(B:" )
       (comment "$B%3%a%s%H(B: ")
       (jisho-comment "$B<-=q(B:%s: $B%3%a%s%H(B:%s")
@@ -416,8 +418,8 @@ Conversion commands
       (register-notify "$A4G5dOnD?!:(B%s$A!;(B(%s: %s)$A=+R*1;5GB<5=(B %s $AVP(B")
       (cannot-remove "$AO5M34G5dOn2;D\O{3}(B")
       (enter-hindo "$AGkJdHkF56H(B: ")
-      (remove-notify "$A4G5dOnD?(B%s(%s)$A=+R*4S(B %s $AVPO{3}(B")
-      (removed "$A4G5dOnD?(B%s(%s)$ARQ>-4S(B%s$AVPO{3}AK(B")
+      (remove-notify "$A4G5dOnD?!:(B%s$A!;(B(%s)$A=+R*4S(B %s $AVPO{3}(B")
+      (removed "$A4G5dOnD?!:(B%s$A!;(B(%s)$ARQ>-4S(B%s$AVPO{3}AK(B")
       (jishomei "$A4G5dC{(B:" )
       (comment "$AW"JM(B: ")
       (jisho-comment "$A4G5d(B:%s: $AW"JM(B:%s")
@@ -481,8 +483,8 @@ Conversion commands
       (register-notify "$(C^vnp(B $(Cz#YM(B $(C!:(B%s$(C!;(B(%s: %s)$(C@;(B %s$(C?!(B $(CTtVbGO0Z@>4O4Y(B.")
       (cannot-remove "$(C=C=:E[(B $(C^vnpz#YM@:(B $(Ca<K[GR(B $(C<v(B $(C>x@>4O4Y(B.")
       (enter-hindo "$(C^:Sx8&(B $(Cl}UtGO=J=C?@(B: ")
-      (remove-notify "$(C^vnpz#YM(B %s(%s)$(C@;(B %s$(C:NEM(B $(Ca<K[GO0Z@>4O4Y(B.")
-      (removed "$(C^vnp(B $(Cz#YM(B %s(%s)$(C@;(B %s$(C:NEM(B $(Ca<K[G_@>4O4Y(B.")
+      (remove-notify "$(C^vnpz#YM!:(B%s$(C!;(B(%s)$(C@;(B %s$(C:NEM(B $(Ca<K[GO0Z@>4O4Y(B.")
+      (removed "$(C^vnp(B $(Cz#YM!:(B%s$(C!;(B(%s)$(C@;(B %s$(C:NEM(B $(Ca<K[G_@>4O4Y(B.")
       (jishomei "$(C^vnpY#(B: " )
       (comment "$(CqI`7(B: ")
       (jisho-comment "$(C^vnp(B:%s: $(CqI`7(B:%s")
@@ -939,13 +941,15 @@ whose name defaults to .eggrc.")
 		      (or (markerp egg:*region-start*)
 			  (setq egg:*region-start* (make-marker)))
 		      (or (markerp egg:*region-end*)
-			  (setq egg:*region-end* (set-marker-insertion-type (make-marker) t)))
+			  (set-marker-insertion-type
+			   (setq egg:*region-end* (make-marker)) t))
 		      (if (null (marker-position egg:*region-start*))
 			  (progn
 			    ;;;(setq egg:*global-map-backup* (current-global-map))
-			    (setq egg:*local-map-backup* (current-local-map))
-			    (and (boundp 'disable-undo) (setq disable-undo t))
+			    ;;(setq egg:*local-map-backup* (current-local-map))
+			    ;;(and (boundp 'disable-undo) (setq disable-undo t))
 			    (delete-region start end)
+			    (suspend-undo)
 			    (goto-char start)
 			    (insert egg:*henkan-open*)
 			    (set-marker egg:*region-start* (point))
@@ -975,10 +979,13 @@ whose name defaults to .eggrc.")
 		      (henkan-goto-bunsetu 0)
 		      ;;;(use-global-map henkan-mode-map)
 		      ;;;(use-local-map nil)
-		      (use-local-map henkan-mode-map)
+		      ;;(use-local-map henkan-mode-map)
 		      (run-hooks 'egg:henkan-start-hook)))))
 	    (setq finished t))
-	(or finished (setq disable-undo nil) (setq egg:henkan-mode-in-use nil)))))
+	(or finished
+	    ;;(setq disable-undo nil)
+	    (setq egg:henkan-mode-in-use nil)
+	    (resume-undo-list)))))
   )
 
 (defun henkan-paragraph ()
@@ -1111,12 +1118,15 @@ optional SHO-BUNSETU-FACE $B$O>.J8@a6h4V$rI=<($9$k(B face $B$^$?$O(B nil"
   (interactive)
   (egg:bunsetu-face-off)
   (egg:henkan-face-off)
+  (setq egg:henkan-mode-in-use nil)
+  (setq egg:*in-fence-mode* nil)
   (delete-region (- egg:*region-start* (length egg:*henkan-open*))
 		 egg:*region-start*)
   (delete-region egg:*region-start* egg:*region-end*)
   (delete-region egg:*region-end* (+ egg:*region-end* (length egg:*henkan-close*)))
   (goto-char egg:*region-start*)
   (setq egg:*sai-henkan-start* (point))
+  (resume-undo-list)
   (let ((i 0) (max (wnn-server-bunsetu-suu)))
     (setq egg:*old-bunsetu-suu* max)
     (while (< i max)
@@ -1127,7 +1137,6 @@ optional SHO-BUNSETU-FACE $B$O>.J8@a6h4V$rI=<($9$k(B face $B$^$?$O(B nil"
       ))
   (setq egg:*sai-henkan-end* (point))
   (wnn-server-hindo-update)
-  (setq egg:henkan-mode-in-use nil)
   (egg:quit-egg-mode)
   (run-hooks 'egg:henkan-end-hook)
   )
@@ -1138,6 +1147,8 @@ optional SHO-BUNSETU-FACE $B$O>.J8@a6h4V$rI=<($9$k(B face $B$^$?$O(B nil"
   (interactive)
   (egg:bunsetu-face-off)
   (egg:henkan-face-off)
+  (setq egg:henkan-mode-in-use nil)
+  (setq egg:*in-fence-mode* nil)
   (delete-region (- egg:*region-start* (length egg:*henkan-open*))
 		 egg:*region-start*)
   (delete-region egg:*region-start* egg:*region-end*)
@@ -1145,6 +1156,7 @@ optional SHO-BUNSETU-FACE $B$O>.J8@a6h4V$rI=<($9$k(B face $B$^$?$O(B nil"
 				     ;; 92.8.5  by Y.Kasai
 				     (length egg:*henkan-close*)))
   (goto-char egg:*region-start*)
+  (resume-undo-list)
   (insert (car (wnn-server-bunsetu-kanji 0)))
   (if (not overwrite-mode)
       (undo-boundary))
@@ -1152,7 +1164,6 @@ optional SHO-BUNSETU-FACE $B$O>.J8@a6h4V$rI=<($9$k(B face $B$^$?$O(B nil"
   (forward-char 1)
   (delete-region (point) egg:*region-end*)
   (wnn-server-hindo-update)
-  (setq egg:henkan-mode-in-use nil)
   (egg:quit-egg-mode)
   )
 ;; end of patch
@@ -1196,7 +1207,7 @@ optional SHO-BUNSETU-FACE $B$O>.J8@a6h4V$rI=<($9$k(B face $B$^$?$O(B nil"
   ;;;(use-global-map fence-mode-map)
   ;;;(use-local-map  nil)
   (setq egg:henkan-mode-in-use nil)
-  (use-local-map fence-mode-map)
+  ;;(use-local-map fence-mode-map)
   (egg:mode-line-display))
 
 ;; ### Should probably put this on a key.
@@ -1210,9 +1221,10 @@ optional SHO-BUNSETU-FACE $B$O>.J8@a6h4V$rI=<($9$k(B face $B$^$?$O(B nil"
 	 (setq egg:henkan-mode-in-use t)
 	 (mode-line-egg-mode-update (egg:msg-get 'henkan-mode-indicator))
 	 (goto-char egg:*sai-henkan-start*)
-	 (setq egg:*local-map-backup* (current-local-map))
-	 (and (boundp 'disable-undo) (setq disable-undo t))
+	 ;;(setq egg:*local-map-backup* (current-local-map))
+	 ;;(and (boundp 'disable-undo) (setq disable-undo t))
 	 (delete-region egg:*sai-henkan-start* egg:*sai-henkan-end*)
+	 (suspend-undo)
 	 (goto-char egg:*sai-henkan-start*)
 	 (insert egg:*henkan-open*)
 	 (set-marker egg:*region-start* (point))
@@ -1223,9 +1235,12 @@ optional SHO-BUNSETU-FACE $B$O>.J8@a6h4V$rI=<($9$k(B face $B$^$?$O(B nil"
 	 (egg:henkan-face-on)
 	 (egg:bunsetu-face-on)
 	 (henkan-goto-bunsetu 0)
-	 (use-local-map henkan-mode-map)
+	 ;;(use-local-map henkan-mode-map)
 	 (setq finished t))
-       (or finished (setq disable-undo nil) (setq egg:henkan-mode-in-use nil)))))
+       (or finished
+	   ;;(setq disable-undo nil)
+	   (setq egg:henkan-mode-in-use nil)
+	   (resume-undo-list)))))
   )
 
 (defun egg:bunsetu-face-on ()
@@ -1272,10 +1287,10 @@ optional SHO-BUNSETU-FACE $B$O>.J8@a6h4V$rI=<($9$k(B face $B$^$?$O(B nil"
 (defun egg:bunsetu-face-off ()
   (and egg:*dai-bunsetu-face*
        (extentp egg:*dai-bunsetu-extent*)
-       (delete-extent egg:*dai-bunsetu-extent*))
+       (detach-extent egg:*dai-bunsetu-extent*))
   (and egg:*sho-bunsetu-face*
        (extentp egg:*sho-bunsetu-extent*)
-       (delete-extent egg:*sho-bunsetu-extent*))
+       (detach-extent egg:*sho-bunsetu-extent*))
   )
 
 (defun henkan-goto-bunsetu (number)
@@ -1467,7 +1482,8 @@ optional SHO-BUNSETU-FACE $B$O>.J8@a6h4V$rI=<($9$k(B face $B$^$?$O(B nil"
   ;;;(use-global-map fence-mode-map)
   ;;;(use-local-map  nil)
   (setq egg:henkan-mode-in-use nil)
-  (use-local-map fence-mode-map)
+  ;;(use-local-map fence-mode-map)
+  (setq egg:*in-fence-mode* t)
   (egg:mode-line-display)
   )
 
@@ -1525,34 +1541,29 @@ optional SHO-BUNSETU-FACE $B$O>.J8@a6h4V$rI=<($9$k(B face $B$^$?$O(B nil"
   (setq unread-command-events (list last-command-event))
   (henkan-kakutei))
 
-(defvar henkan-mode-map (make-keymap))
-(defvar henkan-mode-esc-map (make-keymap))
-
-(define-key henkan-mode-map [t] 'undefined)
-(define-key henkan-mode-esc-map [t] 'undefined)
-
-(let ((ch 0))
-  (while (<= ch 127)
-    (define-key henkan-mode-map (char-to-string ch) 'undefined)
-    (define-key henkan-mode-esc-map (char-to-string ch) 'undefined)
-    (setq ch (1+ ch))))
+(defvar henkan-mode-map (make-sparse-keymap))
+(defvar henkan-mode-esc-map (make-sparse-keymap))
+(set-keymap-default-binding henkan-mode-map 'undefined)
+(set-keymap-default-binding henkan-mode-esc-map 'undefined)
 
 (let ((ch 32))
   (while (< ch 127)
     (define-key henkan-mode-map (char-to-string ch) 'henkan-kakutei-and-self-insert)
     (setq ch (1+ ch))))
 
-(define-key henkan-mode-map "\e"   henkan-mode-esc-map)
-(define-key henkan-mode-map [escape]   henkan-mode-esc-map)
-;(define-key henkan-mode-map "\ei"  'henkan-inspect-bunsetu)
-(define-key henkan-mode-map "\ei"  'henkan-bunsetu-chijime-sho)
-(define-key henkan-mode-map "\eo"  'henkan-bunsetu-nobasi-sho)
-(define-key henkan-mode-map "\es"  'henkan-select-kouho-dai)
-(define-key henkan-mode-map "\eh"  'henkan-hiragana)
-(define-key henkan-mode-map "\ek"  'henkan-katakana)
-(define-key henkan-mode-map "\ez"  'henkan-select-kouho-sho)
-(define-key henkan-mode-map "\e<"  'henkan-saishou-bunsetu)
-(define-key henkan-mode-map "\e>"  'henkan-saichou-bunsetu)
+(condition-case ()
+    (define-key henkan-mode-map "\e" henkan-mode-esc-map)
+  (error nil))
+(define-key henkan-mode-map [escape] henkan-mode-esc-map)
+(define-key henkan-mode-esc-map "i" 'henkan-bunsetu-chijime-sho)
+(define-key henkan-mode-esc-map "o" 'henkan-bunsetu-nobasi-sho)
+(define-key henkan-mode-esc-map "s" 'henkan-select-kouho-dai)
+(define-key henkan-mode-esc-map "h" 'henkan-hiragana)
+(define-key henkan-mode-esc-map "k" 'henkan-katakana)
+(define-key henkan-mode-esc-map "z" 'henkan-select-kouho-sho)
+(define-key henkan-mode-esc-map "<" 'henkan-saishou-bunsetu)
+(define-key henkan-mode-esc-map ">" 'henkan-saichou-bunsetu)
+
 ;(define-key henkan-mode-map " "    'henkan-next-kouho-dai)
 					; 92.9.8 by T.Shingu
 (define-key henkan-mode-map " "    'henkan-next-kouho)
@@ -1565,27 +1576,40 @@ optional SHO-BUNSETU-FACE $B$O>.J8@a6h4V$rI=<($9$k(B face $B$^$?$O(B nil"
 (define-key henkan-mode-map "\C-e" 'henkan-last-bunsetu)
 (define-key henkan-mode-map "\C-f" 'henkan-forward-bunsetu)
 (define-key henkan-mode-map "\C-g" 'henkan-quit)
+(define-key henkan-mode-map "\C-h" 'henkan-help-command)
 (define-key henkan-mode-map "\C-i" 'henkan-bunsetu-chijime-dai)
 (define-key henkan-mode-map "\C-k" 'henkan-kakutei-before-point)
 (define-key henkan-mode-map "\C-l" 'henkan-kakutei)
 (define-key henkan-mode-map "\C-m" 'henkan-kakutei)
-(define-key henkan-mode-map [return] 'henkan-kakutei)
 (define-key henkan-mode-map "\C-n" 'henkan-next-kouho)
 (define-key henkan-mode-map "\C-o" 'henkan-bunsetu-nobasi-dai)
 (define-key henkan-mode-map "\C-p" 'henkan-previous-kouho)
 (define-key henkan-mode-map "\C-t"  'toroku-henkan-mode)
-(define-key henkan-mode-map "\C-u" 'undefined)
 (define-key henkan-mode-map "\C-v" 'henkan-inspect-bunsetu)
 (define-key henkan-mode-map "\C-w" 'henkan-next-kouho-dai)
 (define-key henkan-mode-map "\C-z" 'henkan-next-kouho-sho)
 (define-key henkan-mode-map "\177" 'henkan-quit)
-(define-key henkan-mode-map [delete] 'henkan-quit)
-(define-key henkan-mode-map 'backspace 'henkan-quit)
-(define-key henkan-mode-map '(control h) 'henkan-help-command)
-(define-key henkan-mode-map [right] 'henkan-forward-bunsetu)
-(define-key henkan-mode-map [left] 'henkan-backward-bunsetu)
-(define-key henkan-mode-map [down] 'henkan-next-kouho)
-(define-key henkan-mode-map [up] 'henkan-previous-kouho)
+(define-key henkan-mode-map [backspace] 'henkan-quit)
+(define-key henkan-mode-map [clear]     'henkan-quit)
+(define-key henkan-mode-map [delete]    'henkan-quit)
+(define-key henkan-mode-map [down]      'henkan-next-kouho)
+(define-key henkan-mode-map [help]      'henkan-help-command)
+(define-key henkan-mode-map [kp-enter]  'henkan-kakutei)
+(define-key henkan-mode-map [kp-down]   'henkan-next-kouho)
+(define-key henkan-mode-map [kp-left]   'henkan-backward-bunsetu)
+(define-key henkan-mode-map [kp-right]  'henkan-forward-bunsetu)
+(define-key henkan-mode-map [kp-up]     'henkan-previous-kouho)
+(define-key henkan-mode-map [left]      'henkan-backward-bunsetu)
+(define-key henkan-mode-map [next]      'henkan-next-kouho)
+(define-key henkan-mode-map [prior]     'henkan-previous-kouho)
+(define-key henkan-mode-map [return]    'henkan-kakutei)
+(define-key henkan-mode-map [right]     'henkan-forward-bunsetu)
+(define-key henkan-mode-map [up]        'henkan-previous-kouho)
+
+(unless (assq 'egg:henkan-mode-in-use minor-mode-map-alist)
+  (setq minor-mode-map-alist
+	(cons (cons 'egg:henkan-mode-in-use henkan-mode-map)
+	      minor-mode-map-alist)))
 
 (defun henkan-help-command ()
   "Display documentation for henkan-mode."
@@ -1660,10 +1684,12 @@ optional SHO-BUNSETU-FACE $B$O>.J8@a6h4V$rI=<($9$k(B face $B$^$?$O(B nil"
 	(dict-number
 	 (menu:select-from-menu
 	  (list 'menu (egg:msg-get 'touroku-jishomei)
-		(mapcar '(lambda (x)
-			   (let ((y (car (assoc x dic-list))))
-			     (cons (wnn-dict-name y dic-list) y)))
-			writable-dic-list))))
+		(delq nil
+		      (mapcar '(lambda (x)
+				 (let ((y (car (assoc x dic-list))))
+				   (if y
+				       (cons (wnn-dict-name y dic-list) y))))
+			      writable-dic-list)))))
 	(hinsi-name (hinsi-from-menu dict-number "/"))
 	(hinsi-no (wnn-server-hinsi-number hinsi-name))
 	(dict-name (wnn-dict-name dict-number dic-list)))
@@ -1945,7 +1971,8 @@ Type  x to eXecute the deletions requested.
 "
   )
 
-(defvar diced-mode-map (let ((map (make-sparse-keymap))) (suppress-keymap map) map))
+(defvar diced-mode-map (make-sparse-keymap))
+(set-keymap-default-binding diced-mode-map 'undefined)
 
 (define-key diced-mode-map "a"    'diced-add)
 (define-key diced-mode-map "d"    'diced-delete)
@@ -1983,10 +2010,12 @@ Type  x to eXecute the deletions requested.
 	(dict-number
 	 (menu:select-from-menu
 	  (list 'menu (egg:msg-get 'jishomei)
-		(mapcar '(lambda (x)
-			   (let ((y (assoc x dic-list)))
-			     (cons (nth 1 y) (nth 0 y))))
-			writable-dic-list))))
+		(delq nil
+		      (mapcar '(lambda (x)
+				 (let ((y (assoc x dic-list)))
+				   (if y
+				       (cons (nth 1 y) (nth 0 y)))))
+			      writable-dic-list)))))
 	(comment (read-from-minibuffer (egg:msg-get 'comment)
 				       (wnn-dict-name dict-number dic-list))))
     (if (wnn-server-dict-comment dict-number comment)
